
/******************************************************************************
* MODULE     : segment.hpp
* DESCRIPTION: header for low level vectorial computations
* COPYRIGHT  : (C) 2003  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#ifndef __MML__SEGMENT__HPP
#define __MML__SEGMENT__HPP
#include "base.hpp"

namespace mml {
#define TMPL template<class C>

/******************************************************************************
* Low level vector routines on segments
******************************************************************************/

#define SEGMENT_UNARY(name,op)			\
TMPL void					\
name (C* dest, C* s, nat n) {			\
  while (n != 0) {				\
    *dest = op (*s);				\
    dest++; s++; n--;				\
  }						\
}

#define SEGMENT_BINARY(name,op)			\
TMPL void					\
name (C* dest, C* s1, C* s2, nat n) {		\
  while (n != 0) {				\
    *dest = (*s1) op (*s2);			\
    dest++; s1++; s2++; n--;			\
  }						\
}

#define SEGMENT_INLINE_BINARY(name,op)		\
TMPL void					\
name (C* dest, C* s, nat n) {			\
  while (n != 0) {				\
    (*dest) op (*s);				\
    dest++; s++; n--;				\
  }						\
}

#define SEGMENT_SCALAR(name,op)			\
TMPL void					\
name (C* dest, C* s, C c, nat n) {		\
  while (n != 0) {				\
    *dest = (*s) op (c);			\
    dest++; s++; n--;				\
  }						\
}

#define SEGMENT_INLINE_SCALAR(name,op)		\
TMPL void					\
name (C* dest, C c, nat n) {			\
  while (n != 0) {				\
    (*dest) op (c);				\
    dest++; n--;				\
  }						\
}

#define SEGMENT_TEST(name,nontest)		\
TMPL bool					\
name (C* s1, C* s2, nat n) {			\
  while (n != 0) {				\
    if ((*s1) nontest (*s2)) return false;	\
    s1++; s2++; n--;				\
  }						\
  return true;					\
}

SEGMENT_UNARY(segment_neg,-);
SEGMENT_BINARY(segment_add,+);
SEGMENT_BINARY(segment_sub,-);
SEGMENT_BINARY(segment_mul,*);
SEGMENT_BINARY(segment_div,/);
SEGMENT_BINARY(segment_mod,%);
SEGMENT_SCALAR(segment_mul,*);
SEGMENT_SCALAR(segment_div,/);
SEGMENT_SCALAR(segment_mod,%);
SEGMENT_INLINE_BINARY(segment_copy,=);
SEGMENT_INLINE_BINARY(segment_add,+=);
SEGMENT_INLINE_BINARY(segment_sub,-=);
SEGMENT_INLINE_BINARY(segment_mul,*=);
SEGMENT_INLINE_BINARY(segment_div,/=);
SEGMENT_INLINE_SCALAR(segment_mul,*=);
SEGMENT_INLINE_SCALAR(segment_div,/=);
SEGMENT_INLINE_SCALAR(segment_mod,%=);
SEGMENT_TEST(segment_eq,!=);
SEGMENT_TEST(segment_lesseq,>);
SEGMENT_TEST(segment_gtreq,<);

TMPL void
segment_set (C* dest, C c, nat n) {
  while (n != 0) {
    *dest = c;
    dest++; n--;
  }
}

TMPL void
segment_clear (C* dest, nat n) {
  while (n != 0) {
    *dest = C(0);
    dest++; n--;
  }
}

TMPL void
segment_neg (C* dest, nat n) {
  while (n != 0) {
    *dest = - (*dest);
    dest++; n--;
  }
  return true;
}

TMPL void
segment_mul_add (C* dest, C* s, C c, nat n) {
  while (n != 0) {
    *dest += (*s) * c;
    dest++; s++; n--;
  }
}

#undef TMPL
} // namespace mml
#endif //__MML__SEGMENT__HPP
